@@ -1,9 +1,10 @@
 package Test::Net::RabbitMQ;
-$Test::Net::RabbitMQ::VERSION = '0.11';
 use Moose;
 use warnings;
 use strict;
 
+our $VERSION = '0.12';
+
 # ABSTRACT: A mock RabbitMQ implementation for use when testing.
 
 
@@ -69,7 +70,9 @@ has exchanges => (
 
 has queue => (
     is => 'rw',
-    isa => 'Str'
+    isa => 'Str',
+    predicate => '_has_queue',
+    clearer => '_clear_queue',
 );
 
 has queues => (
@@ -133,6 +136,7 @@ sub connect {
 }
 
 
+my $ctag = 0;
 sub consume {
     my ($self, $channel, $queue, $options) = @_;
 
@@ -151,6 +155,26 @@ sub consume {
     die "no_ack=>0 is not supported at this time" if !$options->{no_ack};
 
     $self->queue($queue);
+
+    return exists $options->{consumer_tag}
+        ? $options->{consumer_tag}
+        : 'consumer-tag-' . $ctag++;
+}
+
+
+sub cancel {
+    my ($self, $channel, $consumer_tag) = @_;
+
+    die "Not connected" unless $self->connected;
+
+    die "You must provide a consumer tag"
+        unless defined $consumer_tag && length $consumer_tag;
+
+    return 0 unless $self->_has_queue;
+
+    $self->_clear_queue;
+
+    return 1;
 }
 
 
@@ -174,6 +198,17 @@ sub exchange_declare {
 }
 
 
+sub exchange_delete {
+    my ($self, $channel, $exchange, $options) = @_;
+
+    die "Not connected" unless $self->connected;
+
+    die "Unknown channel" unless $self->_channel_exists($channel);
+
+    $self->_remove_exchange($exchange);
+}
+
+
 sub tx_select {
     my ($self, $channel) = @_;
 
@@ -277,6 +312,7 @@ sub queue_bind {
 }
 
 
+my $queue = 0;
 sub queue_declare {
     my ($self, $channel, $queue, $options) = @_;
 
@@ -284,7 +320,36 @@ sub queue_declare {
 
     die "Unknown channel: $channel" unless $self->_channel_exists($channel);
 
-    $self->_set_queue($queue, []) unless $self->_queue_exists($queue);
+    if ($options->{passive}) {
+        # Would rabbitmq die if $queue was undef or q{}?
+        return
+               unless defined $queue
+            && length $queue
+            && $self->_queue_exists($queue);
+    }
+    else {
+        $queue = 'queue-' . $queue++
+            unless defined $queue && length $queue;
+        $self->_set_queue($queue, []) unless $self->_queue_exists($queue);
+    }
+
+    return $queue unless wantarray;
+    return (
+        $queue,
+        scalar @{ $self->_get_queue($queue) },
+        $self->queue && $self->queue eq $queue ? 1 : 0,
+    );
+}
+
+
+sub queue_delete {
+    my ($self, $channel, $queue, $options) = @_;
+
+    die "Not connected" unless $self->connected;
+
+    die "Unknown channel" unless $self->_channel_exists($channel);
+
+    $self->_remove_queue($queue);
 }
 
 
@@ -309,6 +374,10 @@ sub publish {
     my $self = shift;
     my $channel = shift;
 
+    die "Not connected" unless $self->connected;
+
+    die "Unknown channel: $channel" unless $self->_channel_exists($channel);
+
     my $messages = $self->_tx_messages->{ $channel };
     if ($messages) {
         push @$messages, [ @_ ];
@@ -321,10 +390,6 @@ sub publish {
 sub _publish {
     my ($self, $channel, $routing_key, $body, $options, $props) = @_;
 
-    die "Not connected" unless $self->connected;
-
-    die "Unknown channel: $channel" unless $self->_channel_exists($channel);
-
     my $exchange = $options->{exchange};
     unless($exchange) {
         $exchange = 'amq.direct';
@@ -364,6 +429,7 @@ sub recv {
 
     $message->{delivery_tag} = $self->_inc_delivery_tag;
     $message->{consumer_tag} = '';
+    $message->{redelivered}  = 0;
 
     return $message;
 }
@@ -398,7 +464,7 @@ Test::Net::RabbitMQ - A mock RabbitMQ implementation for use when testing.
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -420,16 +486,16 @@ version 0.11
     $mq->consume(1, 'new-orders');
 
     my $msg = $mq->recv;
-    
+
     # Or
-    
+
     my $msg = $mq->get(1, 'order.new', {});
 
 =head1 DESCRIPTION
 
 Test::Net::RabbitMQ is a terrible approximation of using the real thing, but
-hopefully will allow you to test systems that use L<Net::RabbitMQ> without
-having to use an actual RabbitMQ instance.
+hopefully will allow you to test systems that use L<Net::AMQP::RabbitMQ> or
+L<Net::RabbitMQ> without having to use an actual RabbitMQ instance.
 
 The general overview is that calls to C<publish> pushes a message into one
 or more queues (or none if there are no bindings) and calls to C<recv>
@@ -437,7 +503,7 @@ pop them.
 
 =head1 CAVEATS
 
-This module has all the features I've needed to successfully test our 
+This module has all the features I've needed to successfully test our
 RabbitMQ-using application. Patches are welcome if I'm missing something you
 need! At the moment there are a number of shortcomings:
 
@@ -459,7 +525,7 @@ If false then any calls to connect will die to emulate a failed connection.
 
 =head2 debug
 
-If set to true (which you can do at any time) then a message will be emmitted
+If set to true (which you can do at any time) then a message will be emitted
 to STDERR any time a message is added to a queue.
 
 =head1 METHODS
@@ -481,6 +547,12 @@ throw an exception if you've set C<connectable> to false.
 
 Sets the queue that will be popped when C<recv> is called.
 
+=head2 cancel($channel, $consumer_tag)
+
+Cancels the subscription for the given consumer tag. Calls to C<recv> after
+this will throw an error unless you call C<consume> again. This method always
+returns true if there is a subscription to cancel, false otherwise.
+
 =head2 disconnect
 
 Disconnects this instance by setting C<connected> to false.
@@ -489,6 +561,10 @@ Disconnects this instance by setting C<connected> to false.
 
 Creates an exchange of the specified name.
 
+=head2 exchange_delete($channel, $exchange, $options)
+
+Deletes an exchange of the specified name.
+
 =head2 tx_select($channel)
 
 Begins a transaction on the specified channel.  From this point forward all
@@ -530,6 +606,10 @@ Only with exact matches of the routing key.>
 
 Creates a queue of the specified name.
 
+=head2 queue_delete($channel, $queue, $options)
+
+Deletes a queue of the specified name.
+
 =head2 queue_unbind($channel, $queue, $exchange, $routing_key)
 
 Unbinds the specified routing key from the provided queue and exchange.
@@ -552,6 +632,7 @@ information:
        routing_key => 'nr_test_q',        # route the message took
        exchange => 'nr_test_x',           # exchange used
        delivery_tag => 1,                 # (inc'd every recv or get)
+       redelivered => $boolean            # if message is redelivered
        consumer_tag => '',                # Always blank currently
        props => $props,                   # hashref sent in
      }
@@ -562,7 +643,7 @@ Cory G Watson <gphat@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Cory G Watson.
+This software is copyright (c) 2015 by Cory G Watson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
